searchData={"items":[{"type":"module","title":"telemetry","doc":"`telemetry` allows you to invoke certain functions whenever a\nparticular event is emitted.\n\nFor more information see the documentation for `attach/4`, `attach_many/4`\nand `execute/2`.","ref":"telemetry.html"},{"type":"function","title":"telemetry.attach/4","doc":"Attaches the handler to the event.\n\n`HandlerId` must be unique, if another handler with the same ID already exists the\n`{error, already_exists}` tuple is returned.\n\nSee `execute/3` to learn how the handlers are invoked.\n\n> #### Function Captures {: .info}\n>\n> Due to how anonymous functions are implemented in the Erlang VM, it is best to use\n> function captures (`fun mod:fun/4` in Erlang or `&Mod.fun/4` in Elixir) as event handlers\n> to achieve the best performance. In other words, avoid using literal anonymous functions\n> (`fun(...) -> ... end` or `fn ... -> ... end`) or local function captures (`fun handle_event/4`\n> or `&handle_event/4`) as event handlers.\n\nAll the handlers are executed by the process dispatching event. If the function fails (raises,\nexits or throws) then the handler is removed and a failure event is emitted.\n\nHandler failure events `[telemetry, handler, failure]` should only be used for monitoring\nand diagnostic purposes. Re-attaching a failed handler will likely result in the handler\nfailing again.\n\nNote that you should not rely on the order in which handlers are invoked.","ref":"telemetry.html#attach/4"},{"type":"function","title":"telemetry.attach_many/4","doc":"Attaches the handler to many events.\n\nThe handler will be invoked whenever any of the events in the `EventNames` list is emitted. Note\nthat failure of the handler on any of these invocations will detach it from all the events in\n`EventNames` (the same applies to manual detaching using `detach/1`).\n\n Note:  due to how anonymous functions are implemented in the Erlang VM, it is best to use\nfunction captures (i.e. `fun mod:fun/4` in Erlang or `&Mod.fun/4` in Elixir) as event handlers\nto achieve maximum performance. In other words, avoid using literal anonymous functions\n(`fun(...) -> ... end` or `fn ... -> ... end`) or local function captures (`fun handle_event/4`\nor `&handle_event/4`) as event handlers.\n\nAll the handlers are executed by the process dispatching event. If the function fails (raises,\nexits or throws) a handler failure event is emitted and then the handler is removed.\n\nHandler failure events `[telemetry, handler, failure]` should only be used for monitoring\nand diagnostic purposes. Re-attaching a failed handler will likely result in the handler\nfailing again.\n\nNote that you should not rely on the order in which handlers are invoked.","ref":"telemetry.html#attach_many/4"},{"type":"function","title":"telemetry.detach/1","doc":"Removes the existing handler.\n\nIf the handler with given ID doesn't exist, `{error, not_found}` is returned.","ref":"telemetry.html#detach/1"},{"type":"function","title":"telemetry.execute/2","doc":"Same as [`execute(EventName, Measurements, #{})`](`execute/3`).","ref":"telemetry.html#execute/2"},{"type":"function","title":"telemetry.execute/3","doc":"Emits the event, invoking handlers attached to it.\n\nWhen the event is emitted, the handler function provided to `attach/4` is called with four\narguments:\n\n  * the event name\n  * the map of measurements\n  * the map of event metadata\n  * the handler configuration given to `attach/4`\n\n#### Best practices and conventions:\n\nWhile you are able to emit messages of any `t:event_name/0` structure, it is recommended that you follow the\nthe guidelines laid out in `span/3` if you are capturing start/stop events.","ref":"telemetry.html#execute/3"},{"type":"function","title":"telemetry.list_handlers/1","doc":"Returns all handlers attached to events with given prefix.\n\nHandlers attached to many events at once using `attach_many/4` will be listed once for each\nevent they're attached to.\nNote that you can list all handlers by feeding this function an empty list.","ref":"telemetry.html#list_handlers/1"},{"type":"function","title":"telemetry.span/3","doc":"Runs the provided `SpanFunction`, emitting start and stop/exception events, invoking the handlers attached to each.\n\nThe `SpanFunction` must return a `{result, stop_metadata}` or a `{result, extra_measurements, stop_metadata}` tuple.\n\nWhen this function is called, 2 events will be emitted via `execute/3`. Those events will be one of the following\npairs:\n\n  * `EventPrefix ++ [start]` and `EventPrefix ++ [stop]`\n  * `EventPrefix ++ [start]` and `EventPrefix ++ [exception]`\n\nHowever, note that in case the current process crashes due to an exit signal\nof another process, then none or only part of those events would be emitted.\nBelow is a breakdown of the measurements and metadata associated with each individual event.\n\nWhen providing `StartMetadata` and `StopMetadata`, these values will be sent independently to `start` and\n`stop` events. If an exception occurs, exception metadata will be merged onto the `StartMetadata`. In general,\nit is **highly recommended** that `StopMetadata` should include the values from `StartMetadata`\nso that handlers, such as those used for metrics, can rely entirely on the `stop` event. Failure to include\nall of `StartMetadata` in `StopMetadata` can add significant complexity to event handlers.\n\nA default span context is added to event metadata under the `telemetry_span_context` key if this key is not provided\nby the user in the `StartMetadata`. This context is useful for tracing libraries to identify unique\nexecutions of span events within a process to match start, stop, and exception events. Metadata keys which\nshould be available to both `start` and `stop` events need to supplied separately for `StartMetadata` and\n`StopMetadata`.\n\nIf `SpanFunction` returns `{result, extra_measurements, stop_metadata}`, then a map of extra measurements\nwill be merged with the measurements automatically provided. This is useful if you want to return, for example,\nbytes from an HTTP request. The standard measurements `duration` and `monotonic_time` cannot be overridden.\n\nFor `telemetry` events denoting the **start** of a larger event, the following data is provided:\n\n  * Event:\n\n    ```\n    EventPrefix ++ [start]\n    ```\n\n  * Measurements:\n\n    ```\n    #{\n      % The current system time in native units from\n      % calling: erlang:system_time()\n      system_time => integer(),\n      monotonic_time => integer(),\n    }\n    ```\n\n  * Metadata:\n\n    ```\n    #{\n      telemetry_span_context => term(),\n      % User defined metadata as provided in StartMetadata\n      ...\n    }\n    ```\n\n\n\nFor `telemetry` events denoting the **stop** of a larger event, the following data is provided:\n\n  * Event:\n\n    ```\n    EventPrefix ++ [stop]\n    ```\n\n  * Measurements:\n\n    ```\n    #{\n      % The current monotonic time minus the start monotonic time in native units\n      % by calling: erlang:monotonic_time() - start_monotonic_time\n      duration => integer(),\n      monotonic_time => integer(),\n      % User defined measurements when returning `SpanFunction` as a 3 element tuple\n    }\n    ```\n\n  * Metadata:\n\n    ```\n    #{\n      % An optional error field if the stop event is the result of an error\n      % but not necessarily an exception.\n      error => term(),\n      telemetry_span_context => term(),\n      % User defined metadata as provided in StopMetadata\n      ...\n    }\n    ```\n\nFor `telemetry` events denoting an **exception** of a larger event, the following data is provided:\n\n  * Event:\n\n    ```\n    EventPrefix ++ [exception]\n    ```\n\n  * Measurements:\n\n    ```\n    #{\n      % The current monotonic time minus the start monotonic time in native units\n      % by calling: erlang:monotonic_time() - start_monotonic_time\n      duration => integer(),\n      monotonic_time => integer()\n    }\n    ```\n\n  * Metadata:\n\n    ```\n    #{\n      kind => throw | error | exit,\n      reason => term(),\n      stacktrace => list(),\n      telemetry_span_context => term(),\n      % User defined metadata as provided in StartMetadata\n       ...\n    }\n    ```","ref":"telemetry.html#span/3"},{"type":"type","title":"telemetry.event_measurements/0","doc":"","ref":"telemetry.html#t:event_measurements/0"},{"type":"type","title":"telemetry.event_metadata/0","doc":"","ref":"telemetry.html#t:event_metadata/0"},{"type":"type","title":"telemetry.event_name/0","doc":"","ref":"telemetry.html#t:event_name/0"},{"type":"type","title":"telemetry.event_prefix/0","doc":"","ref":"telemetry.html#t:event_prefix/0"},{"type":"type","title":"telemetry.event_value/0","doc":"","ref":"telemetry.html#t:event_value/0"},{"type":"type","title":"telemetry.handler/0","doc":"","ref":"telemetry.html#t:handler/0"},{"type":"type","title":"telemetry.handler_config/0","doc":"","ref":"telemetry.html#t:handler_config/0"},{"type":"type","title":"telemetry.handler_function/0","doc":"","ref":"telemetry.html#t:handler_function/0"},{"type":"type","title":"telemetry.handler_id/0","doc":"","ref":"telemetry.html#t:handler_id/0"},{"type":"type","title":"telemetry.span_function/0","doc":"","ref":"telemetry.html#t:span_function/0"},{"type":"type","title":"telemetry.span_result/0","doc":"","ref":"telemetry.html#t:span_result/0"},{"type":"module","title":"telemetry_test","doc":"Functions for testing execution of Telemetry events.\n\nTesting that the correct Telemetry events are emitted with the\nright measurements and metadata is essential for library authors.\nIt helps to maintain stable APIs and avoid accidental changes\nto events.","ref":"telemetry_test.html"},{"type":"function","title":"telemetry_test.attach_event_handlers/2","doc":"Attaches a \"message\" handler to the given events.\n\nThe attached handler sends a message to `DestinationPID` every time it handles one of the\nevents in `events`. The function returns a reference that you can use to make sure that\nmessages come from this handler. This reference is also used as the handler ID, so you\ncan use it to detach the handler with `telemetry:detach/1`.\n\nThe shape of messages sent to `DestinationPID` is:\n\n<!-- tabs-open -->","ref":"telemetry_test.html#attach_event_handlers/2"},{"type":"function","title":"Erlang - telemetry_test.attach_event_handlers/2","doc":"```erlang\n{Event, Ref, Measurements, Metadata}\n```","ref":"telemetry_test.html#attach_event_handlers/2-erlang"},{"type":"function","title":"Elixir - telemetry_test.attach_event_handlers/2","doc":"```elixir\n{event, ref, measurements, metadata}\n```\n\n<!-- tabs-close -->","ref":"telemetry_test.html#attach_event_handlers/2-elixir"},{"type":"function","title":"Examples - telemetry_test.attach_event_handlers/2","doc":"<!-- tabs-open -->","ref":"telemetry_test.html#attach_event_handlers/2-examples"},{"type":"function","title":"Erlang - telemetry_test.attach_event_handlers/2","doc":"An example of a test in Erlang (using [`ct`](https://www.erlang.org/docs/23/man/ct)) could\nlook like this:\n\n```erlang\nRef = telemetry_test:attach_event_handlers(self(), [[some, event]]),\nfunction_that_emits_the_event(),\nreceive\n    {[some, event], Ref, #{measurement := _}, #{meta := _}} ->\n        telemetry:detach(Ref)\nafter 1000 ->\n    ct:fail(timeout_receive_attach_event_handlers)\nend.\n```","ref":"telemetry_test.html#attach_event_handlers/2-erlang"},{"type":"function","title":"Elixir - telemetry_test.attach_event_handlers/2","doc":"An example of an ExUnit test in Elixir could look like this:\n\n```elixir\nref = :telemetry_test.attach_event_handlers(self(), [[:some, :event]])\nfunction_that_emits_the_event()\nassert_received {[:some, :event], ^ref, %{measurement: _}, %{meta: _}}\n```\n\n<!-- tabs-close -->","ref":"telemetry_test.html#attach_event_handlers/2-elixir"},{"type":"extras","title":"Telemetry","doc":"# Telemetry\n\n[![Codecov](https://codecov.io/gh/beam-telemetry/telemetry/branch/master/graphs/badge.svg)](https://codecov.io/gh/beam-telemetry/telemetry/branch/master/graphs/badge.svg)\n\n[Documentation](https://hexdocs.pm/telemetry/)\n\nTelemetry is a lightweight library for dynamic dispatching of events, with a focus on\nmetrics and instrumentation. Any Erlang or Elixir library can use `telemetry` to emit\nevents. Application code and other libraries can then hook into those events and run\ncustom handlers.\n\n> Note: this library is agnostic to tooling and therefore is not directly related to\n> OpenTelemetry. For OpenTelemetry in the Erlang VM, see\n> [opentelemetry-erlang](https://github.com/open-telemetry/opentelemetry-erlang), and check\n> [opentelemetry_telemetry](https://github.com/open-telemetry/opentelemetry-erlang-contrib/tree/main/utilities/opentelemetry_telemetry)\n> to connect both libraries.","ref":"readme.html"},{"type":"extras","title":"Usage - Telemetry","doc":"In a nutshell, you register a custom module and function to be invoked for certain events,\nwhich are executed whenever there is such an event. The event name is a list of atoms. Each event is\ncomposed of a numeric value and can have metadata attached to it. Let's look at an example.\n\nImagine that you have a web application and you'd like to log latency and response status for each\nincoming request. With Telemetry, you can build a module which does exactly that whenever a response\nis sent. The first step is to execute a measurement.\n\nIn Elixir:\n\n```elixir\n:telemetry.execute(\n  [:web, :request, :done],\n  %{latency: latency},\n  %{request_path: path, status_code: status}\n)\n```\n\nIn Erlang:\n\n```erlang\ntelemetry:execute(\n  [web, request, done],\n  #{latency => Latency},\n  #{request_path => Path, status_code => Status}\n)\n```\n\nThen you can create a module to be invoked whenever the event happens.\n\nIn Elixir:\n\n```elixir\ndefmodule LogResponseHandler do\n  require Logger\n\n  def handle_event([:web, :request, :done], measurements, metadata, _config) do\n    Logger.info(\n      \"[#{metadata.request_path}] #{metadata.status_code} sent in #{measurements.latency}\"\n    )\n  end\nend\n```\n\nIn Erlang:\n\n```erlang\n-module(log_response_handler).\n\n-include_lib(\"kernel/include/logger.hrl\").\n\nhandle_event([web, request, done], #{latency := Latency}, #{request_path := Path,\n                                                            status_code := Status}, _Config) ->\n  ?LOG_INFO(\"[~s] ~p sent in ~p\", [Path, Status, Latency]).\n\n```\n\n**Important note:**\n\nThe `handle_event` callback of each handler is invoked synchronously on each `telemetry:execute` call.\nTherefore, it is extremely important to avoid blocking operations. If you need to perform any action\nthat is not immediate, consider offloading the work to a separate process (or a pool of processes)\nby sending a message.\n\nFinally, all you need to do is to attach the module to the executed event.\n\nIn Elixir:\n\n```elixir\n:ok =\n  :telemetry.attach(\n    # unique handler id\n    \"log-response-handler\",\n    [:web, :request, :done],\n    &LogResponseHandler.handle_event/4,\n    nil\n  )\n```\n\nIn Erlang:\n\n```erlang\nok = telemetry:attach(\n  %% unique handler id\n  <<\"log-response-handler\">>,\n  [web, request, done],\n  fun log_response_handler:handle_event/4,\n  []\n)\n```\n\nYou might think that it isn't very useful, because you could just as well write a log statement\ninstead of calling `telemetry:execute/3` – and you would be right! But now imagine that each Elixir library\nwould publish its own set of events with information useful for introspection. Currently each library\nrolls their own instrumentation layer – Telemetry aims to provide a single interface for these use\ncases across the whole ecosystem.","ref":"readme.html#usage"},{"type":"extras","title":"Spans - Telemetry","doc":"In order to provide uniform events that capture the start and end of discrete events, it is recommended\nthat you use the `telemetry:span/3` call. This function will generate a start event and a stop or exception\nevent depending on whether the provided function executed successfully or raised an error. Under the hood,\nthe `telemetry:span/3` function leverages the `telemetry:execute/3` function, so all the same usage patterns\napply. If an exception does occur, an `EventPrefix ++ [exception]` event will be emitted and the caught error\nwill be re-raised.\n\nThe measurements for the `EventPrefix ++ [start]` event will contain a key called `system_time` which is\nderived by calling `erlang:system_time/0`. For `EventPrefix ++ [stop]` and `EventPrefix ++ [exception]`\nevents, the measurements will contain a key called `duration` and its value is derived by calling\n`erlang:monotonic_time() - StartMonotonicTime`. All events include a `monotonic_time` measurement too.\nAll of them represent time as native units.\n\nTo convert the duration from native units you can use:\n\n```elixir\nmilliseconds = System.convert_time_unit(duration, :native, :millisecond)\n```\n\nTo create span events you would do something like this:\n\nIn Elixir:\n\n```elixir\ndef process_message(message) do\n  start_metadata = %{message: message}\n\n  result =\n    :telemetry.span(\n      [:worker, :processing],\n      start_metadata,\n      fn ->\n        result = ... # Process the message\n        {result, %{metadata: \"Information related to the processing of the message\"}}\n      end\n    )\nend\n```\n\nIn Erlang:\n\n```erlang\nprocess_message(Message) ->\n  StartMetadata =  #{message => Message},\n  Result = telemetry:span(\n    [worker, processing],\n    StartMetadata,\n    fun() ->\n      Result = % Process the message\n      {Result, #{metadata => \"Information related to the processing of the message\"}}\n    end\n  ).\n```\n\nTo then attach to the events that `telemetry:span/3` emits you would do the following:\n\nIn Elixir:\n\n```elixir\n:ok =\n  :telemetry.attach_many(\n    \"log-response-handler\",\n    [\n      [:worker, :processing, :start],\n      [:worker, :processing, :stop],\n      [:worker, :processing, :exception]\n    ],\n    &LogResponseHandler.handle_event/4,\n    nil\n  )\n```\n\nIn Erlang:\n\n```erlang\nok = telemetry:attach_many(\n  <<\"log-response-handler\">>,\n  [\n    [worker, processing, start],\n    [worker, processing, stop],\n    [worker, processing, exception]\n  ],\n  fun log_response_handler:handle_event/4,\n  []\n)\n```\n\nWith the following event handler module defined:\n\nIn Elixir:\n\n```elixir\ndefmodule LogResponseHandler do\n  require Logger\n\n  def handle_event(event, measurements, metadata, _config) do\n    Logger.info(\"Event: #{inspect(event)}\")\n    Logger.info(\"Measurements: #{inspect(measurements)}\")\n    Logger.info(\"Metadata: #{inspect(metadata)}\")\n  end\nend\n```\n\nIn Erlang:\n\n```erlang\n-module(log_response_handler).\n\n-include_lib(\"kernel/include/logger.hrl\").\n\nhandle_event(Event, Measurements, Metadata, _Config) ->\n  ?LOG_INFO(\"Event: ~p\", [Event]),\n  ?LOG_INFO(\"Measurements: ~p\", [Measurements]),\n  ?LOG_INFO(\"Metadata: ~p\", [Metadata]).\n```","ref":"readme.html#spans"},{"type":"extras","title":"Installation - Telemetry","doc":"Telemetry is available on [Hex](https://hex.pm/packages/telemetry). To install, just add it to\nyour dependencies in `mix.exs`:\n\n```elixir\ndefp deps() do\n  [\n    {:telemetry, \"~> 1.0\"}\n  ]\nend\n```\n\nor `rebar.config`:\n\n```erlang\n{deps, [{telemetry, \"~> 1.0\"}]}.\n```","ref":"readme.html#installation"},{"type":"extras","title":"Copyright and License - Telemetry","doc":"Copyright (c) 2019 Erlang Ecosystem Foundation and Erlang Solutions.\n\nTelemetry's source code is released under the Apache License, Version 2.0.\n\nSee the [LICENSE](LICENSE) and [NOTICE](NOTICE) files for more information.","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.3.0](https://github.com/elixir-telemetry/telemetry/tree/v1.3.0)","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Ability to return extra measurements from `telemetry:span/3`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Rewrite docs from edoc to OTP 27 `-moduledoc`/`-doc`.\n\n  Internal macros `?DOC` and `?MODULEDOC` are used. They are no-ops prior to OTP 27.\n\n## [1.2.1](https://github.com/elixir-telemetry/telemetry/tree/v1.2.0)","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed a local handler warning log when using `telemetry_test`. (#124)\n\n## [1.2.0](https://github.com/elixir-telemetry/telemetry/tree/v1.2.0)","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added `telemetry_test` module for testing telemetry events. (#118)\n\n## [1.1.0](https://github.com/elixir-telemetry/telemetry/tree/v1.1.0)","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Added `monotonic_time` measurement to all span events. (#92)\n- Added a `[telemetry, handler, failure]` event emitted when any handler fails and is detached. (#98)\n- Added a `mix.exs` file, lowering the memory footprint of compilation in Elixir projects. (#103)\n\n## [1.0.0](https://github.com/elixir-telemetry/telemetry/tree/v1.0.0)\n\nThere are no changes in the 1.0.0 release - it marks the stability of the API.\n\n## [0.4.3](https://github.com/elixir-telemetry/telemetry/tree/v0.4.3)\n\nThis release improves the `telemetry:span/3` function by adding the `telemetry_span_context` metadata\nto all span events. The new metadata enables correlating span events that belong to the same span.","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Added `telemetry_span_context` metadata to all events emitted by `telemetry:span/3`.\n\n## [0.4.2](https://github.com/elixir-telemetry/telemetry/tree/v0.4.2)","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Added the `telemetry:span/3` function.\n\n## [0.4.1](https://github.com/elixir-telemetry/telemetry/tree/v0.4.1)\n\nApart from the code changes listed below, this release includes a few improvements\nto the documentation.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Calls to `execute/3` when `telemetry` application is not started no longer cause an error.\n\n## [0.4.0](https://github.com/elixir-telemetry/telemetry/tree/v0.4.0)\n\nA single event value has been replaced by a map of measurements. Now it is up to the consumer of the\nevent to decide what part of the payload is important. This is useful in cases where event indicates\nthat a thing happened but there are many properties describing it. For example, a database query\nevent may include total time, decode time, wait time and other measurements.","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- `execute/3` now accepts a map of measurements instead of event value","ref":"changelog.html#changed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- `:telemetry.execute/3` with an event value in favour of `:telemetry.execute/3` with a map of\n  measurements. If the event value is provided, it is put in a map under a `:value` key and provided\n  as measurements to a handler function.\n\n## [0.3.0](https://github.com/elixir-telemetry/telemetry/tree/v0.3.0)\n\nThis release marks the conversion from Elixir to Erlang. This is a breaking change, but the benefits\nlargely surpass the drawbacks - Telemetry core can now be used by all projects running on the BEAM,\nregardless of the language they're written in.","ref":"changelog.html#deprecated"},{"type":"extras","title":"Added - Changelog","doc":"- Added `:telemetry.handler/0`, `:telemetry.handler_function/0` and `:telemetry.handler_config/0`\n  types.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The library has been rewritten to Erlang. In Elixir, `:telemetry` module has to be used in place\n  of `Telemetry`. In Erlang, `telemetry` module has to be used in place of `'Elixir.Telemetry'`;\n- `:telemetry.list_handlers/1` returns a list of maps (of type `:telemetry.handler/0`) instead of\n  a list of tuples;\n- `:telemetry.attach/4` and `:telemetry.attach_many/4` replaced the 5-arity versions and now accept\n  an anonymous function for the handler function instead of a module and function name.","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed `:telemetry.attach/5` and `:telemetry.attach_many/5` - 4-arity versions need to be used\n  now instead.\n\n## [0.2.0](https://github.com/elixir-telemetry/telemetry/tree/v0.2.0)\n\nThe main point of this release is to mark base Telemetry API as stable, so that other libraries can\nrely on it without worrying about backwards compatibility.","ref":"changelog.html#removed"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed `Telemetry.attach/4` and `Telemetry.attach_many/4` - the handler config is now required.","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed type specs which were producing Dialyzer errors.\n\n## [0.1.0](https://github.com/elixir-telemetry/telemetry/tree/v0.1.0)\n\nFirst release of Telemetry library.","ref":"changelog.html#fixed"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"},{"type":"extras","title":"NOTICE","doc":"Copyright (c) 2018, Chris McCord and Erlang Solutions\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"notice.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.34.2"}}